// -*-C++-*-
#ifndef SIMPLE_FITS_H
#define SIMPLE_FITS_H

#include <string>
#include <map>
#include <iostream>

#include <fitsio.h>

#include "Exception/Exception.h"

namespace SimpleFits {

    class File {

    public:

	fitsfile *fptr;

        File( ) : fptr( NULL), status( 0 ) {};
	File( const std::string& file );
	~File( void );

        bool read_keyword( const char*, double& value);
	double read_keyword_double( const char* keyname );
	double read_keyword_double( std::string& keyname ) {
	    return read_keyword_double( keyname.c_str() );
	}

        std::string read_keyword( const char* keyname );
	std::string read_keyword( const std::string& keyname ) {
	    return read_keyword( keyname.c_str() );
	}

    protected:
	int status;
	std::string    file;
	Exception exception( std::string pfx = "" );
    };

    //

    class Table : public File {

    public:

	struct Column {

	    int      typecode;
	    int      colnum;
	    std::string   name;
	    LONGLONG repeat;
	    std::string   unit;
	    long     width;

	    Column() {};
	    Column( Table& tbl, const std::string& alias, const std::string& name );

	};

	long num_rows( void ) { return _nrows; }
	long rowsize( void )  { return _rowsize; }
	LONGLONG read_col_double( double* &dst, const Column& col, LONGLONG start_row, LONGLONG nrows );


	Column column( const std::string& alias, const std::string& colname ) {

	    return Column( *this, alias, colname );
	}

	bool exists_column( const std::string& column_name );

	Table( const std::string& file );
    private:

	long _nrows;
	long _rowsize;
    };

    std::ostream& operator<<( std::ostream& ostr, const Table::Column& c );

}

#endif // ! FITS_FILE_H
